/******************************************************************************/
/*                                                                            */
/*  Title       : satread.c                                                   */
/*  Author      : Manfred Bester, DL5KR                                       */
/*  Date        : 03Mar92                                                     */
/*  Last change : 14Oct92                                                     */
/*                                                                            */
/*  Synopsis    : Routines for reading data from resource files and other     */
/*                I/O functions.                                              */
/*                                                                            */
/*  Resources   : sites.dat, defaults.dat, modes.dat, nnn                     */
/*                                                                            */
/*  sites.dat:    contains information on various gorund stations             */
/*                column  1-24:  ground station name                          */
/*                       26-35:  latitude  [deg] (north = "+")                */
/*                       37-47:  longitude [deg] (west  = "+")                */
/*                       49-55:  altitude  [m]                                */
/*                                                                            */
/*  defaults.dat: contains default parameters                                 */
/*                line 1, column 1-24:  ground station name                   */
/*                line 2, column 1-13:  satellite name                        */
/*                line 3, column 1- 7:  element filename                      */
/*                line 4, column 1- 3:  time zone designator (e.g. UTC, PST)  */
/*                line 5, column 1- 2:  time Zone [h]                         */
/*                line 6, column 1- 4:  duration for orbit prediction [d]     */
/*                line 7, column 1- 4:  minimum elevation [deg]               */
/*                                                                            */
/*  modes.dat:    contains mode information for amateur satellites            */
/*                                                                            */
/*  nnnn:         contains NORAD two-line element sets                        */
/*                (e.g. 933, STS, TVRO)                                       */
/*                                                                            */
/******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <math.h>

#include "satglobalsx.h"
#include "satglobals.h"
#include "sattrack.h"
#include "vt100.h"

extern modeRec modes[MAXMODES];

/******************************************************************************/
/*                                                                            */
/* clipString: clips N characters plus leading blanks off the beginning of a  */
/*             character string                                               */
/*                                                                            */
/******************************************************************************/

void clipString(charStr,N)

int  N;
char *charStr;

{
    int  i, j, firstChar;
    char tmpStr[100];

    firstChar = FALSE;
    j = 0;

    for (i = 0; i < strlen(charStr)-N; i++) 
    {

       if (charStr[i+N] != ' ' || (charStr[i+N] == ' ' && firstChar))
       {
           tmpStr[j] = charStr[i+N];
           j++;
           firstChar = TRUE;
       }
    }

    tmpStr[j] = '\0';                            /* add termination character */
    sprintf(charStr,"%s",tmpStr);
}

/******************************************************************************/
/*                                                                            */
/* getTimeZone: gets time zone [h] from the time zone string information      */
/*                                                                            */
/******************************************************************************/

void getTimeZone()

{
    int i;

    timeZone = 0.0;

    for (i = 0; i < timeZones; i++)
    {
        if (!strcmp(timeZoneList[i],timeZoneStr))
            timeZone = timeZoneHour[i] / 24.0;          /* gets time zone [d] */
    }

    if (timeZone == 0.0)        /* set to UTC if time zone is specified wrong */
        sprintf(timeZoneStr,"UTC");
}

/******************************************************************************/
/*                                                                            */
/* getDefaultData: reads default parameters from 'defaults.dat'               */
/*                                                                            */
/******************************************************************************/

void getDefaultData()

{
    char defaultLine[120], defaultDuration[60], defaultMinElev[60];
    FILE *defaultFile;

    sprintf(defaultsDat,"/usr1/%s/SatTrack/Data/%s",SATUSER,DEFAULTS);

    if ((defaultFile = fopen(defaultsDat,"r")) == 0)
    {
        printf("cannot open input file '%s'\n",defaultsDat);
        exit(-1);
    }

    sprintf(defaultSite,"Zero Lat Long");
    sprintf(defaultSat,"Mir");
    sprintf(defaultSet,"000");
    sprintf(defaultTimeZoneStr,"UTC");
    sprintf(defaultDuration,"1.0");
    sprintf(defaultMinElev,"0.0");

    while (TRUE)
    {
        fgets(defaultLine,100,defaultFile);
        truncBlanks(defaultLine);

        if (!strncmp(defaultLine,"Site:",5))
        {
           clipString(defaultLine,5);
           sprintf(defaultSite,"%s",defaultLine);
        }

        if (!strncmp(defaultLine,"Satellite:",10))
        {
           clipString(defaultLine,10);
           sprintf(defaultSat,"%s",defaultLine);
        }

        sscanf(defaultLine,"Element set: %s",defaultSet);
        sscanf(defaultLine,"Element set type: %s",defaultType);
        sscanf(defaultLine,"Time zone: %s",defaultTimeZoneStr);
        sscanf(defaultLine,"Duration: %s",defaultDuration);
        sscanf(defaultLine,"Min elevation: %s",defaultMinElev);

        if (!strlen(defaultLine))
            break;                                             /* EOF */

        if (strlen(defaultLine) <= 2)
            break;                                             /* blank line */
    }

    strcpy(timeZoneStr,defaultTimeZoneStr);
    getTimeZone();
    defDuration     = atof(defaultDuration);
    defMinElevation = atof(defaultMinElev);

    fclose(defaultFile);

    if (verbose)
    {
        printf("Ground station   : %17s\n",defaultSite);
        printf("Satellite        : %17s\n",defaultSat);
        printf("Element set      : %17s\n",defaultSet);
        printf("Element set type : %17s\n",defaultType);
        printf("Time zone        : %17s (%+.0f)\n",timeZoneStr,timeZone*24.0);
        printf("Duration         : %17.1f d\n",defDuration);
        printf("Min elevation    : %17.1f deg\n",defMinElevation);
        nl();
    }
}

/******************************************************************************/
/*                                                                            */
/* getSatFileName: gets reduced satellite name for creating output file, i.e. */
/*                 replace blanks in file name with "_", delete "(" and ")"   */
/*                                                                            */
/******************************************************************************/

void getSatFileName()

{
    int i, j;

    i = 0; j = 0;

    do
    {
        if (satName[i] != ' ' && satName[i] != '(' && 
            satName[i] != ')' && satName[i] != '"')
        {
            satFileName[j] = satName[i];
            i++; j++;
        }

        if (satName[i] == ' ')
        {
            satFileName[j] = '_';
            i++; j++;
        }

        if (satName[i] == '(' || satName[i] == ')' || satName[i] == '"')
        {
            i++;
        }
    }
    while (satName[i] != '\0');

    satFileName[i] = '\0';
}

/******************************************************************************/
/*                                                                            */
/* getSatParams: reads satellite parameters from NORAD two-line data file     */
/*                                                                            */
/******************************************************************************/

void getSatParams()

{
    double epochDayTime, epochSec;
    int    epochYear, epochMonth, epochMonthDay, epochHour, epochMin, error;
    char   stsName[20];

    error = TRUE;

    while (error)
    {
        printf("Satellite name        <%13s> : ",defaultSat);
        gets(satName);

        if (!strlen(satName))
            sprintf(satName,"%s",defaultSat);

        strcpy(stsName,satName);
        upperCase(stsName);
        stsFlag = strncmp(stsName,"STS",3) ? FALSE : TRUE;

        if (stsFlag)
            sprintf(defaultNORAD,"%s",stsName);
        else
            sprintf(defaultNORAD,"%s",defaultSet);

        printf("Two-line element file       <%7s> : ",defaultNORAD);
        gets(elementFile);

        if (stsFlag)
        {
            printf("Element set type              <%5s> : ",defaultType);
            gets(elementType);

            if (!strlen(elementType))
                sprintf(elementType,"%s",defaultType);
            else
                upperCase(elementType);
        }

        else
            sprintf(elementType,"NORAD");

        if (!strlen(elementFile))
            sprintf(elementFile,"%s",defaultNORAD);

        error = readnorad(SATUSER,elementFile,satName,
                   &satNum,&elementSet,&epochDay,&inclination,&epochRAAN,
                   &eccentricity,&epochArgPerigee,&epochMeanAnomaly,
                   &epochMeanMotion,&decayRate,&epochOrbitNum);

        if (error)
        {
            nl(); reverseblink();

            if (error == 1) 
                printf(" data file '%s' not available \n",elementFile);
            if (error == 2)
                printf(" check sum error in line 1, continuing anyway \n");
            if (error == 3)
                printf(" check sum error in line 2, continuing anyway \n");
            if (error == 4)
                printf(" satellite '%s' not listed in data file '%s' \n",
                        satName,elementFile);

            normal(); alarm(); nl();

            if (error == 2 || error == 3) error = FALSE; /* reset to continue */
        }

        getSatFileName();                               /* remove blanks etc. */
    }

    elsetEpochDay     = epochDay;
    epochYear         = (int) (epochDay  / 1000.0);
    epochDay         -= epochYear * 1000.0;
    elsetEpoch        = epochDay + getDayNum(epochYear,1,0);
    epochDayTime      = epochDay - (int) epochDay;

    calendar(epochYear,(int) epochDay,&epochMonth,&epochMonthDay);

    epochDay         += getDayNum(epochYear,1,0);

    epochHour         = (int) (epochDayTime*24.0);
    epochMin          = (int) ((epochDayTime*24.0 - epochHour)*60.0);
    epochSec          = ((epochDayTime*24.0 - epochHour)*60.0 - epochMin)*60.0;

    inclination      *= CDR;
    epochRAAN        *= CDR;
    epochArgPerigee  *= CDR;
    epochMeanAnomaly *= CDR;

    if (verbose)
    {
        printf("Satellite name   : %16s\n",satName);
        printf("Satellite number : %16ld\n",satNum);
        printf("Element set      : %16d\n",elementSet);
        printf("Epoch            : %16.9f d        ",elsetEpochDay);
        printf("%02d-%02d-%02d  %02d:%02d:%06.3f UTC\n",
                epochMonthDay,epochMonth,epochYear,epochHour,epochMin,epochSec);
        printf("Mean anomaly     : %16.9f deg\n",epochMeanAnomaly * CRD);
        printf("Arg of perigee   : %16.9f deg\n",epochArgPerigee * CRD);
        printf("RAAN             : %16.9f deg\n",epochRAAN * CRD);
        printf("Inclination      : %16.9f deg\n",inclination * CRD);
        printf("Eccentricity     : %16.9f\n",eccentricity);
        printf("Mean motion      : %16.9f rev/d\n",epochMeanMotion);
        printf("Decay rate       : %16.9f rev/d^2\n",decayRate);
        printf("Orbit            : %16ld\n",epochOrbitNum);
        printf("\n");
    }

    else
    {
        printf("Epoch: %02d-%02d-%02d  %02d:%02d:%06.3f UTC\n\n",
                epochMonthDay,epochMonth,epochYear,epochHour,epochMin,epochSec);
    }
}

/******************************************************************************/
/*                                                                            */
/* getSatModes: reads modes of the specified satellite                        */
/*                                                                            */
/******************************************************************************/

void getSatModes()

{
    int  foundMode;
    char modeLine[120];
    FILE *modeFile;

    beaconFreq   = BEACON;
    attLong      = 0.0;
    attLat       = 0.0;
    perigeePhase = 0.0;
    maxPhase     = MAXPHASE;
    numModes     = 0;
    launchFlag   = FALSE;

    sprintf(satAlias,"");

    sprintf(modesDat,"/usr1/%s/%s/%s",SATUSER,DATA,MODES);

    if ((modeFile = fopen(modesDat,"r")) != 0)
    {
        foundMode  = FALSE;

        while (!foundMode)
        {
            fgets(modeLine,100,modeFile);

            if (!strlen(modeLine))
                break;                                                 /* EOF */

            truncBlanks(modeLine);
            sprintf(string,"Satellite: %s",satName);

            if (!strncmp(modeLine,string,strlen(string)))
                foundMode = TRUE;
        }

        if (foundMode)
        {
            while (TRUE)
            {
                fgets(modeLine,100,modeFile);

                if (!strlen(modeLine))
                    break;                                             /* EOF */

                if (strlen(modeLine) <= 2)
                    break;                                      /* blank line */

                sscanf(modeLine,"Alias: %s",satAlias);
                sscanf(modeLine,"Beacon: %lf",&beaconFreq);

                launchFlag = (sscanf(modeLine,"Launch: %d-%d-%d %d:%d:%d",
                       &launchDay,&launchMonth,&launchYear,
                       &launchHour,&launchMin,&launchSec)) ? TRUE : FALSE;

                sscanf(modeLine,"Perigee phase: %lf",&perigeePhase);
                sscanf(modeLine,"Attitude: %lf %lf",&attLong,&attLat);
                sscanf(modeLine,"Max phase: %lf",&maxPhase);

                if (sscanf(modeLine,"Mode: %20s %d - %d",
                    modes[numModes].modeStr,&modes[numModes].minPhase,
                    &modes[numModes].maxPhase) == 3 && numModes < MAXMODES)
                    numModes++;
            }
        }

        fclose(modeFile);
    }
}

/******************************************************************************/
/*                                                                            */
/* getSiteParams: reads site parameters from input file 'sites.dat'           */
/*                                                                            */
/******************************************************************************/
 
void getSiteParams()

{
    int  i, siteNameLen, foundSite;
    char siteLine[120];
    FILE *siteFile;

    foundSite = FALSE;

    while (!foundSite)
    { 
        printf("Ground station    <%17s> : ",defaultSite);
        gets(siteName);
 
        if (!strlen(siteName))
            sprintf(siteName,"%s",defaultSite);

        sprintf(sitesDat,"/usr1/%s/%s/%s",SATUSER,DATA,SITES);

        if ((siteFile = fopen(sitesDat,"r")) == 0)
        {
            printf("cannot open input file '%s'\n",sitesDat);
            exit(-1);
        }
 
        siteNameLen = strlen(siteName);
        upperCase(siteName);

        while (fgets(siteLine,100,siteFile) && !foundSite)
        {
            strncpy(string,siteLine,siteNameLen);
            upperCase(string);

            if (strncmp(string,siteName,siteNameLen) == 0)
            {
                foundSite = TRUE;
                strncpy(siteName,siteLine,18);
                truncBlanks(siteName);

                siteLat  = getElement(siteLine,26,35) * CDR;
                siteLong = getElement(siteLine,37,47) * CDR;
                siteAlt  = getElement(siteLine,49,55) * CMKM;

                if (verbose)
                {
                    nl();
                    printf("Ground station   : %17s\n",siteName);
                    printf("Latitude         : %17.6f deg N\n",siteLat * CRD);
                    printf("Longitude        : %17.6f deg W\n",siteLong * CRD);
                    printf("Altitude         : %17.6f m\n",siteAlt / CMKM);
                    nl();
                }

                else
                {
                    if (strcmp(siteName,defaultSite))
                        printf("%s\n",siteName);
                }
            }
        }

        fclose(siteFile);

        if (!foundSite)
        {
            nl(); reverseblink();
            printf(" no matching ground station name \n");
            normal(); alarm(); nl();
        }
    }
}
 
/******************************************************************************/
/*                                                                            */
/* getTimeParams: reads additional parameters for orbit calculations          */
/*                                                                            */
/******************************************************************************/

void getTimeParams()

{
    double hour;
    int    sysYear, sysMonth, sysDay, sysYearDay, sysHour, sysMin, sysSec;
    char   dummys[40], displayString[10];
 
    getUnixTime(&sysDay,&sysMonth,&sysYear,&sysYearDay,
                &sysHour,&sysMin,&sysSec);

    printf("Start time (%02d-%02d-%02d  %02d:%02d:%02d)   <Y> ? ",
            sysDay,sysMonth,sysYear,sysHour,sysMin,sysSec);

    gets(displayString);
    upperCase(displayString);

    if (strlen(displayString) && strncmp(displayString,YES,1))
    {
        printf("Start date (UTC)           [dd mm yy] : ");
        scanf("%d %d %d",&sysDay,&sysMonth,&sysYear);
        printf("Start time (UTC)           [hh mm ss] : ");
        scanf("%d %d %d",&sysHour,&sysMin,&sysSec);

        gets(dummys);                                         /* empty buffer */
    }

    defStepTime = 1.0 / epochMeanMotion * SPD / 150.0;
    defStepTime = (double) (((int) (defStepTime / 10.0)) * 10.0);

    if (!shortPredFlag)
    {
        printf("Time step [s]                  <%4.0f> : ",defStepTime);
        gets(displayString);

        if (!strlen(displayString))
            stepTime = defStepTime;
        else
            stepTime = atof(displayString);
    }

    printf("Duration [d]                   <%4.1f> : ",defDuration);
    gets(displayString);

    if (!strlen(displayString))
        duration = defDuration;
    else
        duration = atof(displayString);

    startTime  = (double) getDayNum(sysYear,sysMonth,sysDay);          /* [d] */
    hour       = (double) (sysHour + sysMin/60.0 + sysSec/3600.0);     /* [h] */
    startTime += hour / 24.0;
    stepTime  /= SPD;                                                  /* [d] */
    stopTime   = startTime + duration;                                 /* [d] */

    if (!shortPredFlag)
    {
        printf("Minimum elevation [deg]        <%4.1f> : ",defMinElevation);
        gets(displayString);

        if (!strlen(displayString))
            minElevation = defMinElevation * CDR;
        else
            minElevation = atof(displayString) * CDR;
    }
}
 
/******************************************************************************/
/*                                                                            */
/* printMode: prints phase-dependent mode of satellite into output file       */
/*                                                                            */
/******************************************************************************/

void printMode(outFile,phase)

{
    int curMode;
 
    for (curMode = 0; curMode < numModes; curMode++)
    {
        if ((phase >= modes[curMode].minPhase
            && phase < modes[curMode].maxPhase)
            || ((modes[curMode].minPhase > modes[curMode].maxPhase)
            && (phase >= modes[curMode].minPhase
            || phase < modes[curMode].maxPhase)))
        {
            fprintf(outFile," %2s",modes[curMode].modeStr);
        }
    }

    if (numModes == 0)
        fprintf(outFile,"   ");
}
 
/******************************************************************************/
/*                                                                            */
/* upperCase: changes lower to upper case letters                             */
/*                                                                            */
/******************************************************************************/

void upperCase(string)

char *string;

{
    int i;

    for (i = 0; i < strlen(string); i++)
    {
        if (string[i] >= 'a' && string[i] <= 'z')
            string[i] -= 'a' - 'A';
    }
}

/******************************************************************************/
/*                                                                            */
/* truncBlanks: truncates trailing blanks from character string               */
/*                                                                            */
/******************************************************************************/

void truncBlanks(string)

char *string;

{
    int i;

    i = strlen(string) - 1;

    do
    {
        i--;
    }
    while (string[i] == ' ');

    string[i+1] = '\0';                          /* add termination character */
}

/******************************************************************************/
/*                                                                            */
/* End of function block satread.c                                            */
/*                                                                            */
/******************************************************************************/
